/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */

package freemule.application;

import java.io.File;
import java.io.IOException;
import java.io.RandomAccessFile;

/**
 * A small, file-based cache for freenet keys.
 * <p>
 * <strong>Note:</strong> The use of this cache is currently disabled in the
 * {@link EngineService}. I believe that though it might be useful for net (in
 * terms of not overloading it with requests) it might prove harmful to the
 * distribution of announcement keys.
 * 
 * @author <a href="mailto:droden@gmail.com">David Roden </a>
 * @version $Id: Cache.java 130 2006-02-18 18:27:11Z bombe $
 */
public class Cache {

	/**
	 * Whether this cache is currently enabled.
	 */
	private boolean enabled = true;

	/**
	 * The directory to store cached data.
	 */
	private File dataDirectory;

	/**
	 * The directory to store cached metadata.
	 */
	private File metadataDirectory;

	/**
	 * Creates a new Cache in <code>baseDirectory</code>. In this directory
	 * two new directories are created, one for cached data, the other one for
	 * cached metadata.
	 * 
	 * @param baseDirectory
	 *            The directory to create the cache in
	 */
	public Cache(String baseDirectory) {
		this(new File(baseDirectory));
	}

	/**
	 * Creates a new Cache in <code>baseDirectory</code>. In this directory
	 * two new directories are created, one for cached data, the other one for
	 * cached metadata.
	 * 
	 * @param baseDirectory
	 *            The directory to create the cache in
	 */
	public Cache(File baseDirectory) {
		this.dataDirectory = new File(baseDirectory, "data");
		this.metadataDirectory = new File(baseDirectory, "metadata");
		if (!dataDirectory.exists()) {
			dataDirectory.mkdirs();
		}
		if (!metadataDirectory.exists()) {
			metadataDirectory.mkdirs();
		}
	}

	/**
	 * Enables or disables this cache.
	 * 
	 * @param enabled
	 *            <code>true</code> to enable, <code>false</code> to disable
	 */
	public void setEnabled(boolean enabled) {
		this.enabled = enabled;
	}

	/**
	 * Returns whether this cache is currently enabled.
	 * 
	 * @return <code>true</code> if enabled, <code>false</code> otherwise
	 */
	public boolean isEnabled() {
		return enabled;
	}

	/**
	 * Returns whether the cache has the data for the given <code>uri</code>.
	 * 
	 * @param uri
	 *            The URI to get
	 * @return <code>true</code> if the data is in the cache,
	 *         <code>false</code> otherwise
	 */
	public boolean hasKey(String uri) {
		if (!enabled)
			return false;
		File f = new File(dataDirectory, uri);
		return f.exists();
	}

	/**
	 * Returns cached data and metadata for the given <code>uri</code>.
	 * 
	 * @param uri
	 *            The URI to get
	 * @return <code>true</code> if the data is in the cache,
	 *         <code>false</code> otherwise
	 */
	public Data getKey(String uri) {
		if (!enabled)
			return null;
		File f = new File(dataDirectory, uri);
		if (f.exists()) {
			byte[] data = null;
			byte[] metadata = null;
			try {
				RandomAccessFile raf = new RandomAccessFile(f, "r");
				data = new byte[(int) raf.length()];
				raf.readFully(data);
				raf.close();
			} catch (IOException ioe1) {
				System.out.println("CACH: Could not read data from cache for " + uri + ".");
				data = null;
			}
			f = new File(metadataDirectory, uri);
			if (f.exists()) {
				try {
					RandomAccessFile raf = new RandomAccessFile(f, "r");
					metadata = new byte[(int) raf.length()];
					raf.readFully(metadata);
					raf.close();
				} catch (IOException ioe1) {
					System.out.println("CACH: Could not read metadata from cache for " + uri + ".");
					metadata = null;
				}
				if ((data != null) && (metadata != null)) {
					System.out.println("CACH: Got " + uri + " from cache.");
					return new Data(metadata, data);
				}
			}
		}
		return null;
	}

	/**
	 * Stores the given <code>data</code> under the specified <code>uri</code>.
	 * <p>
	 * <strong>Warning:</storing> This method does not recognize SSK keys in
	 * any way, so it is currently not possible to store data using the private
	 * key and retrieving it using the public key!
	 * 
	 * @param uri
	 *            The URI under which to store the data
	 * @param data
	 *            The data to store
	 */
	public void putKey(String uri, Data data) {
		if (!enabled)
			return;
		int p = uri.lastIndexOf("/");
		String dirs = uri.substring(0, p);
		File f = new File(dataDirectory, dirs);
		f.mkdirs();
		f = new File(dataDirectory, uri);
		try {
			RandomAccessFile raf = new RandomAccessFile(f, "rwd");
			raf.write(data.getData());
			raf.close();
		} catch (IOException ioe1) {
			System.out.println("CACH: Could not write data for " + uri + ".");
			return;
		}
		f = new File(metadataDirectory, dirs);
		f.mkdirs();
		f = new File(metadataDirectory, uri);
		try {
			RandomAccessFile raf = new RandomAccessFile(f, "rwd");
			raf.write(data.getMetadata());
			raf.close();
			System.out.println("CACH: Wrote data to cache for " + uri + ".");
		} catch (IOException ioe1) {
			System.out.println("CACH: Could not write metadata for " + uri + ".");
		}
	}

}
