/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */

package freemule.application;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;
import java.util.TimeZone;

import org.w3c.dom.Document;

import freemule.util.DateFormatter;
import freemule.util.Hex;
import freemule.util.SimpleXML;
import freemule.util.XML;
import freenet.fcp.FcpException;
import freenet.fcp.FcpRuntimeException;

/**
 * The Discovery Service looks for discovery information of other clients.
 * 
 * @author <a href="mailto:droden@gmail.com">David Roden </a>
 * @version $Id: DiscoveryService.java 130 2006-02-18 18:27:11Z bombe $
 */
public class DiscoveryService extends AbstractEngineService {

	public static final String NAME = "Discovery Service";

	private List<DiscoveryListener> discoveryListeners;

	private TimeZone gmt = TimeZone.getTimeZone("GMT-00:00");
	private Calendar currentDate = null;
	private int index;
	private int missing;
	private int hoursBack;
	private int maxHoursBack = 2;

	public String getName() {
		return NAME;
	}

	public void addDiscoveryListener(DiscoveryListener discoveryListener) {
		if (!discoveryListeners.contains(discoveryListener)) {
			discoveryListeners.add(discoveryListener);
		}
	}

	public void removeDiscoveryListener(DiscoveryListener discoveryListener) {
		discoveryListeners.remove(discoveryListener);
	}

	/**
	 * Notifies listeners that discovery information is being searched at the
	 * specified key.
	 * 
	 * @param key
	 *            The key of the discovery information
	 */
	private void fireDiscoveryStarted(String key) {
		for (DiscoveryListener discoveryListener: discoveryListeners) {
			discoveryListener.discoveryStarted(key);
		}
	}

	/**
	 * Notifies listeners that discovery information could not be found at the
	 * specified key.
	 * 
	 * @param key
	 *            The key of the discovery information
	 * @param exception
	 *            The exception that prevented finding the discovery information
	 */
	private void fireDiscoveryFailed(String key, Exception exception) {
		for (DiscoveryListener discoveryListener: discoveryListeners) {
			discoveryListener.discoveryFailed(key, exception);
		}
	}

	/**
	 * Notifies listeners that discovery information of a new client was found.
	 * 
	 * @param key
	 *            The key of the discovery information
	 * @param remoteClient
	 *            The new client that was discovered
	 */
	private void fireDiscoverySucceeded(String key, Client remoteClient) {
		for (DiscoveryListener discoveryListener: discoveryListeners) {
			discoveryListener.discoveryComplete(key, remoteClient);
		}
	}

	public void setMaxHoursBack(int maxHoursBack) {
		this.maxHoursBack = maxHoursBack;
	}

	public void init() {
		super.init();
		discoveryListeners = new ArrayList<DiscoveryListener>();
		gmt = TimeZone.getTimeZone("GMT-00:00");
		currentDate = null;
		index = 0;
		missing = 0;
		hoursBack = maxHoursBack;
	}

	public void destroy() {
		discoveryListeners.clear();
		discoveryListeners = null;
		gmt = null;
		currentDate = null;
		super.destroy();
	}

	protected void work() {
		while (!stopping) {
			if (hoursBack == maxHoursBack) {
				currentDate = Calendar.getInstance(gmt);
			}
			String key = "KSK@" + networkName + "/announcement/" + DateFormatter.getKeyPortion(currentDate) + "/" + Hex.toHex(index);
			index++;
			Data data = null;
			try {
				fireDiscoveryStarted(key);
				data = engine.getKey(key, hopsToLive);
			} catch (FcpRuntimeException fre1) {
				fireDiscoveryFailed(key, fre1);
			} catch (FcpException fe1) {
				fireDiscoveryFailed(key, fe1);
			} catch (IOException ioe1) {
				fireDiscoveryFailed(key, ioe1);
			} catch (EngineStoppedException ese1) {
				fireServiceAborted(ese1);
				stopping = true;
			}
			if (data == null) {
				missing++;
				if (missing == 3) {
					currentDate.add(Calendar.HOUR, -1);
					index = 0;
					missing = 0;
					hoursBack--;
					if (hoursBack < 0) {
						hoursBack = maxHoursBack;
					}
				}
			} else {
				Document discovery = readDiscovery(data);
				if (discovery != null) {
					Client remoteClient = parseClient(discovery);
					fireDiscoverySucceeded(key, remoteClient);
				}
				missing = 0;
			}
		}
	}

	private Document readDiscovery(Data data) {
		byte[] discoveryData = data.getData();
		Document discovery = XML.transformToDocument(discoveryData);
		return discovery;
	}

	private Client parseClient(Document clientDocument) {
		SimpleXML clientXML = SimpleXML.fromDocument(clientDocument);
		Client remoteClient = new Client(clientXML.getNode("announcement").getNode("client-id").getValue());
		remoteClient.setName(clientXML.getNode(new String[] { "announcement", "client-name" }).getValue());
		remoteClient.setAddress(clientXML.getNode(new String[] { "announcement", "client-address" }).getValue());
		remoteClient.setVersion(clientXML.getNode(new String[] { "announcement", "client-version" }).getValue());
		remoteClient.setFilelistIndex((int) Hex.toLong(clientXML.getNode(new String[] { "announcement", "client-filelist-index" }).getValue()));
		remoteClient.setTimestamp(Hex.toLong(clientXML.getNode(new String[] { "announcement", "client-timestamp" }).getValue()));
		return remoteClient;
	}

}
