/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */

package freemule.application;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import org.w3c.dom.Document;

import freemule.util.Hex;
import freemule.util.SimpleXML;
import freemule.util.XML;
import freenet.fcp.FcpException;
import freenet.fcp.FcpKeyCollisionException;
import freenet.fcp.FcpRuntimeException;

/**
 * @author <a href="mailto:droden@gmail.com">David Roden</a>
 * @version $Id: FilelistInsertService.java 130 2006-02-18 18:27:11Z bombe $
 */
public class FilelistInsertService extends AbstractEngineService {

	public static final String NAME = "Filelist Insert Service";

	private List<FilelistInsertListener> filelistInsertListeners;

	private String clientPrivateAddress;
	private SharedFile[] sharedFiles = null;
	private FreenetFile[] downloadFiles = null;
	private int index = 0;
	private boolean listChanged = false;

	public String getName() {
		return NAME;
	}

	public void addFilelistInsertListener(FilelistInsertListener filelistInsertListener) {
		if (!filelistInsertListeners.contains(filelistInsertListener)) {
			filelistInsertListeners.add(filelistInsertListener);
		}
	}

	public void removeFilelistInsertListener(FilelistInsertListener filelistInsertListener) {
		filelistInsertListeners.remove(filelistInsertListener);
	}

	/**
	 * Notifies listeners that a filelist insert is about to start.
	 * 
	 * @param key
	 *            The key of the filelist
	 * @param currentIndex
	 *            The current index of the filelist
	 */
	private void fireFilelistInsertStarted(String key, int currentIndex) {
		for (FilelistInsertListener filelistInsertListener: filelistInsertListeners) {
			filelistInsertListener.filelistInsertStarted(key, currentIndex);
		}
	}

	/**
	 * Notifies listeners that a filelist insert failed.
	 * 
	 * @param key
	 *            The key of the filelist
	 * @param currentIndex
	 *            The current index of the filelist
	 * @param exception
	 *            The exception that prevented insertion
	 */
	private void fireFilelistInsertFailed(String key, int currentIndex, Exception exception) {
		for (FilelistInsertListener filelistInsertListener: filelistInsertListeners) {
			filelistInsertListener.filelistInsertFailed(key, currentIndex, exception);
		}
	}

	/**
	 * Notifies listeners that the filelist was successfully inserted.
	 * 
	 * @param key
	 *            The key of the filelist
	 * @param index
	 *            The current index of the filelist
	 */
	private void fireFilelistInsertComplete(String key, int index) {
		for (FilelistInsertListener filelistInsertListener: filelistInsertListeners) {
			filelistInsertListener.filelistInsertComplete(key, index);
		}
	}

	public void setClientPrivateAddress(String clientPrivateAddress) {
		this.clientPrivateAddress = clientPrivateAddress;
	}

	public void setSharedFiles(SharedFile[] sharedFiles) {
		synchronized (syncObject) {
			if (sharedFiles != null) {
				listChanged = true;
			}
			this.sharedFiles = new SharedFile[sharedFiles.length];
			System.arraycopy(sharedFiles, 0, this.sharedFiles, 0, sharedFiles.length);
			syncObject.notify();
		}
	}

	public void setDownloadFiles(FreenetFile[] downloadFiles) {
		synchronized (syncObject) {
			if (downloadFiles != null) {
				listChanged = true;
			}
			this.downloadFiles = new FreenetFile[downloadFiles.length];
			System.arraycopy(downloadFiles, 0, this.downloadFiles, 0, downloadFiles.length);
			syncObject.notify();
		}
	}

	public void setChanged(boolean changed) {
		listChanged = changed;
	}

	public void setIndex(int index) {
		synchronized (syncObject) {
			this.index = index;
		}
	}

	public int getIndex() {
		synchronized (syncObject) {
			return index;
		}
	}

	private int currentIndex = -1;

	public void init() {
		super.init();
		filelistInsertListeners = new ArrayList<FilelistInsertListener>();
		clientPrivateAddress = null;
	}

	public void start() {
		super.start();
		if (!listChanged) {
			fireFilelistInsertComplete(clientPrivateAddress + "/freeMule/filelist/" + Hex.toHex(index), index);
		}
	}

	public void destroy() {
		clientPrivateAddress = null;
		filelistInsertListeners.clear();
		filelistInsertListeners = null;
		super.destroy();
	}

	protected void work() {
		synchronized (syncObject) {
			currentIndex = index;
		}

		String keyPrefix = clientPrivateAddress + "/freeMule/filelist/";
		boolean uploaded;
		synchronized (syncObject) {
			uploaded = !listChanged;
		}
		while (!uploaded) {
			Document filelist;
			synchronized (syncObject) {
				filelist = createFilelist();
				listChanged = false;
			}
			byte[] filelistContent = XML.transformToByteArray(filelist);
			Data filelistData = new Data(new byte[0], filelistContent);

			String key = keyPrefix + Hex.toHex(currentIndex);
			try {
				fireFilelistInsertStarted(key, currentIndex);
				engine.putKey(key, hopsToLive, filelistData);
				uploaded = true;
				synchronized (syncObject) {
					index = currentIndex;
				}
				fireFilelistInsertComplete(key, currentIndex);
			} catch (FcpKeyCollisionException fkce1) {
				currentIndex++;
			} catch (FcpRuntimeException fre1) {
				fireFilelistInsertFailed(key, currentIndex, fre1);
			} catch (IOException ioe1) {
				fireFilelistInsertFailed(key, currentIndex, ioe1);
			} catch (FcpException fe1) {
				fireServiceAborted(fe1);
				stopping = true;
				break;
			} catch (EngineStoppedException ese1) {
				fireServiceAborted(ese1);
				stopping = true;
				break;
			}
		}
	}

	private Document createFilelist() {
		SimpleXML filelistXML = new SimpleXML("filelist");
		SimpleXML filesXML = filelistXML.append("files");
		for (SharedFile sharedFile: sharedFiles) {
			SimpleXML fileXML = filesXML.append("file");
			fileXML.append("name", sharedFile.getName());
			fileXML.append("length", Hex.toHex(sharedFile.getLength()));
			fileXML.append("key", sharedFile.getKey());
			fileXML.append("comment", sharedFile.getComment());
			fileXML.append("rating", Hex.toHex(sharedFile.getRating()));
		}
		return filelistXML.getDocument();
	}

}
