/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */

package freemule.application;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

/**
 * Manages filelists for clients.
 * 
 * @author <a href="mailto:droden@gmail.com">David Roden</a>
 * @version $Id: FilelistService.java 130 2006-02-18 18:27:11Z bombe $
 */
public class FilelistService extends AbstractService {

	public static final String NAME = "Filelist Service";
	/**
	 * The {@link Set list} of registered {@link FilelistListener}s.
	 */
	private Set<FilelistListener> filelistListeners;
	/**
	 * The filelists of clients.
	 */
	private Map<Client, SharedFile[]> clientFilelists;

	/**
	 * {@inheritDoc}
	 */
	public String getName() {
		return NAME;
	}

	/**
	 * Adds a {@link FilelistListener} to the list of registered listeners.
	 * 
	 * @param filelistListener
	 *            The {@link FilelistListener} to add
	 */
	public void addFilelistListener(FilelistListener filelistListener) {
		if (!filelistListeners.contains(filelistListener)) {
			filelistListeners.add(filelistListener);
		}
	}

	/**
	 * Removes a {@link FilelistListener} from the list of registered listeners.
	 * 
	 * @param filelistListener
	 *            The {@link FilelistListener} to remove
	 */
	public void removeFilelistListener(FilelistListener filelistListener) {
		filelistListeners.remove(filelistListener);
	}

	/**
	 * Notifies listeners that a new filelist has been added.
	 * 
	 * @param remoteClient
	 *            The client whose filelist was added
	 * @param sharedFiles
	 *            The filelist that was added
	 */
	private void fireFilelistAdded(Client remoteClient, SharedFile[] sharedFiles) {
		for (FilelistListener filelistListener: filelistListeners) {
			filelistListener.filelistAdded(remoteClient, sharedFiles);
		}
	}

	/**
	 * Notifies listeners that a filelist has been updated.
	 * 
	 * @param remoteClient
	 *            The client whose filelist was updated
	 * @param sharedFiles
	 *            The updated filelist
	 */
	private void fireFilelistUpdated(Client remoteClient, SharedFile[] sharedFiles) {
		for (FilelistListener filelistListener: filelistListeners) {
			filelistListener.filelistUpdated(remoteClient, sharedFiles);
		}
	}

	/**
	 * Notifies listeners that a filelist was removed.
	 * 
	 * @param remoteClient
	 *            The client whose filelist was removed
	 */
	private void fireFilelistRemoved(Client remoteClient) {
		for (FilelistListener filelistListener: filelistListeners) {
			filelistListener.filelistRemoved(remoteClient);
		}
	}

	/**
	 * Adds a filelist of a client.
	 * 
	 * @param remoteClient
	 *            The client whose filelist to add
	 * @param sharedFiles
	 *            The filelist to add
	 */
	public void addClientFilelist(Client remoteClient, SharedFile[] sharedFiles) {
		boolean isUpdate = false;
		synchronized (this) {
			isUpdate = clientFilelists.containsKey(remoteClient);
			clientFilelists.put(remoteClient, sharedFiles);
		}
		if (isUpdate) {
			fireFilelistUpdated(remoteClient, sharedFiles);
		} else {
			fireFilelistAdded(remoteClient, sharedFiles);
		}
	}

	/**
	 * Removes a filelist of a client.
	 * 
	 * @param remoteClient
	 *            The client whose filelist to remove
	 */
	public void removeClientFilelist(Client remoteClient) {
		synchronized (this) {
			clientFilelists.remove(remoteClient);
		}
		fireFilelistRemoved(remoteClient);
	}

	/**
	 * Returns the filelist of a client.
	 * 
	 * @param client
	 *            The client whose filelist to get
	 * @return The filelist of the client, or <code>null</code> if the
	 *         filelist does not exist
	 */
	public synchronized SharedFile[] getClientFilelist(Client client) {
		return clientFilelists.get(client);
	}

	/**
	 * Returns a list of all clients that currently have a filelist.
	 * 
	 * @return A list of all clients that currently have a filelist
	 */
	public synchronized Client[] getClients() {
		Set<Client> clients = clientFilelists.keySet();
		return clients.toArray(new Client[clients.size()]);
	}

	/**
	 * {@inheritDoc}
	 */
	public void init() {
		super.init();
		clientFilelists = new HashMap<Client, SharedFile[]>();
		filelistListeners = new HashSet<FilelistListener>();
	}

	/**
	 * {@inheritDoc}
	 */
	public void destroy() {
		clientFilelists.clear();
		clientFilelists = null;
		filelistListeners.clear();
		filelistListeners = null;
		super.destroy();
	}

}
