/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */

package freemule.application;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.BitSet;
import java.util.Properties;

import freemule.io.FilePartInputStream;
import freemule.io.FilePartOutputStream;
import freemule.util.Hex;

/**
 * An extension of {@link FreenetFile} that supports segmentation and check
 * blocks.
 * 
 * @author <a href="mailto:droden@gmail.com">David Roden</a>
 * @version $Id: FreenetFile.java 130 2006-02-18 18:27:11Z bombe $
 */
public class FreenetFile {

	public static final int BLOCKSIZE = 1048576;

	/**
	 * The name of the file.
	 */
	protected String name;

	/**
	 * The key of the file, or <code>null</code> if the key is not yet known.
	 */
	protected String key;

	/**
	 * The length of the file, or <code>-1</code> if the length is not yet
	 * known.
	 */
	protected long length = -1;

	/**
	 * The name of the local file that stores the data of the FreenetFile.
	 */
	protected String dataFilename;

	/**
	 * The name of the file that stores the properties.
	 */
	protected String propertiesFilename;

	/**
	 * The keys for the data blocks.
	 */
	protected String[] blockKeys;

	/**
	 * The status for the manifest.
	 */
	protected BlockStatus manifestStatus;

	/**
	 * The status for the data blocks.
	 */
	protected BlockStatus[] blockStatus;

	/**
	 * The status of the file.
	 */
	protected FileStatus status;

	public FreenetFile() {
		super();
	}

	/**
	 * Creates a new {@link FreenetFile} with the same properties as
	 * <code>freenetFile</code>.
	 * 
	 * @param freenetFile
	 *            The file to copy
	 */
	public FreenetFile(FreenetFile freenetFile) {
		this.name = freenetFile.name;
		this.key = freenetFile.key;
		this.dataFilename = freenetFile.dataFilename;
		this.status = freenetFile.status;
		this.manifestStatus = freenetFile.manifestStatus;
		setLength(freenetFile.length);
	}

	/**
	 * Returns the key of the file, or <code>null</code> if the key is not yet
	 * known.
	 * 
	 * @return The key of the file, or <code>null</code>
	 */
	public synchronized String getKey() {
		return key;
	}

	/**
	 * Sets the key of the file. <code>key</code> can be null to signal that
	 * the key is not yet known.
	 * 
	 * @param key
	 *            The key to set, or <code>null</code>
	 */
	public synchronized void setKey(String key) {
		this.key = key;
	}

	/**
	 * Returns the length of the file, or <code>-1</code> if the length is not
	 * yet known.
	 * 
	 * @return The length of the file, or <code>-1</code>
	 */
	public synchronized long getLength() {
		return length;
	}

	/**
	 * Sets the length of the file. <code>length</code> can be <code>-1</code>
	 * to signal that the length is now yet known.
	 * 
	 * @param length
	 *            The length to set, or <code>-1</code>
	 */
	public synchronized void setLength(long length) {
		this.length = length;
		int blockCount = getBlockCount();
		blockKeys = new String[blockCount];
		blockStatus = new BlockStatus[blockCount];
		for (int blockIndex = 0; blockIndex < blockCount; blockIndex++) {
			blockKeys[blockIndex] = null;
			blockStatus[blockIndex] = BlockStatus.notAvailable;
		}
	}

	/**
	 * Returns the name of the file.
	 * 
	 * @return The name of the file
	 */
	public synchronized String getName() {
		return name;
	}

	/**
	 * Sets the name of the file.
	 * 
	 * @param name
	 *            The name to set
	 */
	public synchronized void setName(String name) {
		this.name = name;
	}

	/**
	 * Returns the name of the file that holds the data of this file.
	 * 
	 * @return The name of the file that holds the data of this file
	 */
	public synchronized String getDataFilename() {
		return dataFilename;
	}

	/**
	 * Sets the name of the file that holds the data of this file.
	 * 
	 * @param dataFilename
	 *            The name of the file that holds the data of this file
	 */
	public synchronized void setDataFilename(String dataFilename) {
		this.dataFilename = dataFilename;
	}

	/**
	 * Returns an {@link InputStream} that returns the data of the file.
	 * 
	 * @return An {@link InputStream} that returns the data of the file
	 * @throws FileNotFoundException
	 *             if the data file does not exist
	 */
	public synchronized InputStream getDataInputStream() throws FileNotFoundException {
		return new FileInputStream(new File(dataFilename));
	}

	/**
	 * Returns an {@link OutputStream} that writes to the data file.
	 * <p>
	 * <strong>Warning:</strong> the data file will be overwritten!
	 * 
	 * @return An {@link OutputStream} that writes to the data file.
	 * @throws FileNotFoundException
	 *             if the data file can not be created
	 */
	public synchronized OutputStream getDataOutputStream() throws FileNotFoundException {
		return new FileOutputStream(new File(dataFilename));
	}

	/**
	 * Sets the {@link FileStatus} of this file.
	 * 
	 * @param fileStatus
	 *            The {@link FileStatus} to set
	 */
	public synchronized void setStatus(FileStatus fileStatus) {
		this.status = fileStatus;
	}

	/**
	 * Returns the {@link FileStatus} of this file.
	 * 
	 * @return The {@link FileStatus} of this file
	 */
	public synchronized FileStatus getStatus() {
		return status;
	}

	/**
	 * @return Returns the manifestStatus.
	 */
	public synchronized BlockStatus getManifestStatus() {
		return manifestStatus;
	}

	/**
	 * @param manifestStatus
	 *            The manifestStatus to set.
	 */
	public synchronized void setManifestStatus(BlockStatus manifestStatus) {
		this.manifestStatus = manifestStatus;
	}

	/**
	 * Returns the filename of the properties file.
	 * 
	 * @return The filename of the properties file
	 */
	public String getPropertiesFilename() {
		return propertiesFilename;
	}

	/**
	 * Sets the filename of the properties file.
	 * 
	 * @param propertiesFilename
	 *            The filename of the properties file
	 */
	public void setPropertiesFilename(String propertiesFilename) {
		this.propertiesFilename = propertiesFilename;
	}

	public void setBlockKey(int blockNumber, String key) {
		blockKeys[blockNumber] = key;
	}

	public String getBlockKey(int blockNumber) {
		return blockKeys[blockNumber];
	}

	public void setBlockStatus(int blockIndex, BlockStatus blockStatus) {
		this.blockStatus[blockIndex] = blockStatus;
	}

	public BlockStatus getBlockStatus(int blockIndex) {
		return blockStatus[blockIndex];
	}
	
	public BitSet getBlockList() {
		return getBlockList(BlockStatus.available);
	}
	
	public BitSet getBlockList(BlockStatus desiredStatus) {
		BitSet resultBitSet = new BitSet(getBlockCount());
		for (int blockIndex = 0, blockCount = getBlockCount(); blockIndex < blockCount; blockIndex++) {
			if (blockStatus[blockIndex] == desiredStatus) {
				resultBitSet.set(blockIndex);
			}
		}
		return resultBitSet;
	}

	/**
	 * Returns an {@link InputStream} that streams a single data block.
	 * 
	 * @param blockNumber
	 *            The number of the block
	 * @return An {@link InputStream} that streams the specified block
	 * @throws FileNotFoundException
	 *             if {@link #dataFilename} can not be found
	 * @throws IOException
	 *             if an I/O error occurs
	 */
	public synchronized InputStream getBlockInputStream(int blockNumber) throws FileNotFoundException, IOException {
		long blockOffset = blockNumber * BLOCKSIZE;
		return new FilePartInputStream(dataFilename, blockOffset, Math.min(BLOCKSIZE, length - blockOffset));
	}

	/**
	 * Returns an {@link OutputStream} that streams to position in the data file
	 * that belongs to the specified block.
	 * 
	 * @param blockNumber
	 *            The number of the block
	 * @return An {@link OutputStream} that will dump the data to the correct
	 *         place on the disk
	 * @throws FileNotFoundException
	 *             if {@link #dataFilename} can not be created
	 * @throws IOException
	 *             if an I/O error occurs
	 */
	public synchronized OutputStream getBlockOutputStream(int blockNumber) throws FileNotFoundException, IOException {
		long blockOffset = blockNumber * BLOCKSIZE;
		return new FilePartOutputStream(dataFilename, blockOffset, Math.min(BLOCKSIZE, length - blockOffset));
	}

	/**
	 * Checks whether <code>o</code> is a FreenetFile and has the same key as
	 * this FreenetFile.
	 * 
	 * @param o
	 *            The object to check for equality
	 * @return <code>true</code> if <code>o</code> is a FreenetFile and has
	 *         the same key as this FreenetFie
	 */
	@Override
	public boolean equals(Object o) {
		if ((o == null) || !(o instanceof FreenetFile)) {
			return false;
		}
		FreenetFile freenetFile = (FreenetFile) o;
		return freenetFile.key.equals(key);
	}

	/**
	 * Returns the hash code of this FreenetFile's key.
	 * 
	 * @see String#hashCode()
	 * @return The hash code of this FreenetFile's key
	 */
	@Override
	public int hashCode() {
		return key.hashCode();
	}

	/**
	 * Creates a {@link Properties} object from this file. All property names
	 * are prefixed with <code>prefix</code> and a dot (<code>'.'</code>).
	 * The resulting {@link Properties} object can be used to re-create this
	 * file using {@link #createFromProperties(Properties, String)}.
	 * 
	 * @param prefix
	 *            The prefix for the property names
	 * @return A {@link Properties} object with this file's properties
	 */
	public Properties getProperties(String prefix) {
		Properties fileProperties = new Properties();
		fileProperties.setProperty(prefix + ".Name", name);
		fileProperties.setProperty(prefix + ".Key", key);
		fileProperties.setProperty(prefix + ".Length", Hex.toHex(length));
		fileProperties.setProperty(prefix + ".DataFilename", dataFilename);
		fileProperties.setProperty(prefix + ".PropertiesFilename", propertiesFilename);
		fileProperties.setProperty(prefix + ".Status", Hex.toHex(status.ordinal()));
		fileProperties.setProperty(prefix + ".ManifestStatus", Hex.toHex(manifestStatus.ordinal()));
		for (int blockIndex = 0, blockCount = getBlockCount(); blockIndex < blockCount; blockIndex++) {
			String blockPrefix = prefix + ".Block." + Hex.toHex(blockIndex);
			if (blockKeys[blockIndex] != null) {
				fileProperties.setProperty(blockPrefix + ".Key", blockKeys[blockIndex]);
			}
			fileProperties.setProperty(blockPrefix + ".Status", Hex.toHex(blockStatus[blockIndex].ordinal()));
		}
		return fileProperties;
	}

	/**
	 * Creates a FreenetFile from the properties stored in
	 * <code>fileProperties</code>. All property names have to be prefixed by
	 * <code>prefix</code> in the same way {@link #getProperties(String)} does
	 * it. If the property named <code><i>prefix</i>.Name</code> does not
	 * exist, <code>null</code> is returned.
	 * 
	 * @param fileProperties
	 *            The properties object to create a FreenetFile from
	 * @param prefix
	 *            The prefix of the property names
	 * @return The created FreenetFile, or <code>null</code>
	 */
	public static FreenetFile createFromProperties(Properties fileProperties, String prefix) {
		FreenetFile freenetFile = new FreenetFile();
		if (!fileProperties.containsKey(prefix + ".Name")) {
			return null;
		}
		freenetFile.setName(fileProperties.getProperty(prefix + ".Name"));
		freenetFile.setKey(fileProperties.getProperty(prefix + ".Key"));
		freenetFile.setLength(Hex.toLong(fileProperties.getProperty(prefix + ".Length")));
		freenetFile.setDataFilename(fileProperties.getProperty(prefix + ".DataFilename"));
		freenetFile.setPropertiesFilename(fileProperties.getProperty(prefix + ".PropertiesFilename"));
		freenetFile.setStatus(FileStatus.values()[(int) Hex.toLong(fileProperties.getProperty(prefix + ".Status"))]);
		freenetFile.setManifestStatus(BlockStatus.values()[(int) Hex.toLong(fileProperties.getProperty(prefix + ".ManifestStatus"))]);
		return freenetFile;
	}

	/**
	 * @return Returns the blockCount.
	 */
	public int getBlockCount() {
		return (int) (((length - 1) / BLOCKSIZE) + 1);
	}

}
