/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */

package freemule.application;

import java.io.IOException;
import java.util.ArrayList;
import java.util.BitSet;
import java.util.Calendar;
import java.util.List;
import java.util.TimeZone;

import freemule.util.DateFormatter;
import freemule.util.Hex;
import freemule.util.SimpleXML;
import freemule.util.XML;
import freenet.fcp.FcpException;
import freenet.fcp.FcpRuntimeException;

/**
 * @author <a href="mailto:droden@gmail.com">David Roden</a>
 * @version $Id: UploadRequestDiscoveryService.java,v 1.1 2005/11/28 22:50:39
 *          bombe Exp $
 */
public class UploadRequestDiscoveryService extends AbstractEngineService {

	public static final String NAME = "Upload Request Discovery Service";

	private List<UploadRequestDiscoveryListener> uploadRequestDiscoveryListeners;

	private int hoursBack = 24;
	private int gapSize = 3;

	private TimeZone gmt;
	private Calendar currentDate;
	private int index;
	private int missing;
	private int hours;

	public String getName() {
		return NAME;
	}

	public void addUploadRequestDiscoveryListener(UploadRequestDiscoveryListener uploadRequestDiscoveryListener) {
		if (!uploadRequestDiscoveryListeners.contains(uploadRequestDiscoveryListener)) {
			uploadRequestDiscoveryListeners.add(uploadRequestDiscoveryListener);
		}
	}

	public void removeUploadRequestListener(UploadRequestInsertListener uploadRequestListener) {
		uploadRequestDiscoveryListeners.remove(uploadRequestListener);
	}

	/**
	 * Notifies listeners that the discovery of an upload request is about to
	 * start.
	 * 
	 * @param key
	 *            The key of the upload request
	 */
	private void fireUploadRequestDiscoveryStarted(String key) {
		for (UploadRequestDiscoveryListener uploadRequestDiscoveryListener: uploadRequestDiscoveryListeners) {
			uploadRequestDiscoveryListener.uploadRequestDiscoveryStarted(key);
		}
	}

	/**
	 * Notifies listeners that the discovery of an upload request failed.
	 * 
	 * @param key
	 *            The key of the upload request
	 * @param exception
	 *            The exception that prevented discovery
	 */
	private void fireUploadRequestDiscoveryFailed(String key, Exception exception) {
		for (UploadRequestDiscoveryListener uploadRequestDiscoveryListener: uploadRequestDiscoveryListeners) {
			uploadRequestDiscoveryListener.uploadRequestDiscoveryFailed(key, exception);
		}
	}

	/**
	 * Notifies listeners that the discovery of an upload request is complete.
	 * 
	 * @param key
	 *            The key of the upload request
	 */
	private void fireUploadRequestDiscoveryComplete(String key, UploadRequest uploadRequest) {
		for (UploadRequestDiscoveryListener uploadRequestDiscoveryListener: uploadRequestDiscoveryListeners) {
			uploadRequestDiscoveryListener.uploadRequestDiscoveryComplete(key, uploadRequest);
		}
	}

	public void init() {
		super.init();
		uploadRequestDiscoveryListeners = new ArrayList<UploadRequestDiscoveryListener>();
		gmt = TimeZone.getTimeZone("GMT-00:00");
		currentDate = null;
		index = 0;
		missing = 0;
		hours = hoursBack;
	}

	public void destroy() {
		uploadRequestDiscoveryListeners.clear();
		uploadRequestDiscoveryListeners = null;
		gmt = null;
		currentDate = null;
		super.destroy();
	}

	protected void work() {
		while (!stopping) {
			if (hoursBack == hours) {
				currentDate = Calendar.getInstance(gmt);
			}

			String key = "KSK@" + networkName + "/uploadrequest/" + DateFormatter.getKeyPortion(currentDate) + "/" + Hex.toHex(index);
			index++;
			Data data = null;
			try {
				fireUploadRequestDiscoveryStarted(key);
				data = engine.getKey(key, hopsToLive);
			} catch (FcpRuntimeException fre1) {
				fireUploadRequestDiscoveryFailed(key, fre1);
			} catch (IOException ioe1) {
				fireUploadRequestDiscoveryFailed(key, ioe1);
			} catch (FcpException fe1) {
				fireServiceAborted(fe1);
				stopping = true;
			} catch (EngineStoppedException ese1) {
				fireServiceAborted(ese1);
				stopping = true;
			}
			if (data == null) {
				missing++;
				if (missing == gapSize) {
					currentDate.add(Calendar.HOUR, -1);
					index = 0;
					missing = 0;
					hoursBack--;
					if (hoursBack < 0) {
						hoursBack = hours;
					}
				}
			} else {
				SimpleXML uploadRequestXML = SimpleXML.fromDocument(XML.transformToDocument(data.getData()));
				UploadRequest uploadRequest = createUploadRequest(uploadRequestXML);
				fireUploadRequestDiscoveryComplete(key, uploadRequest);
				missing = 0;
			}
		}
	}

	private UploadRequest createUploadRequest(SimpleXML uploadRequestXML) {
		String clientId = uploadRequestXML.getNode("upload-request").getNode("client-id").getValue();
		long timestamp = Hex.toLong(uploadRequestXML.getNode("upload-request").getNode("timestamp").getValue());
		SimpleXML[] requestNodes = uploadRequestXML.getNode("upload-request").getNode("requests").getNodes("request");
		UploadRequest uploadRequest = new UploadRequest(clientId, timestamp);
		for (SimpleXML requestNode: requestNodes) {
			String key = requestNode.getNode("key").getValue();
			BitSet blockList = Hex.toBitSet(requestNode.getNode("blocks").getValue());
			uploadRequest.addKey(key, blockList);
		}
		return uploadRequest;
	}

}
