/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */

package freemule.application;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * @author David Roden &lt;droden@gmail.com&gt;
 * @version $Id: UploadRequestService.java 130 2006-02-18 18:27:11Z bombe $
 */
public class UploadRequestService extends AbstractService {

	public static final String NAME = "Upload Request Service";
	private Set<UploadRequestListener> uploadRequestListeners;
	private Map<Client, UploadRequest> clientUploadRequests;

	/**
	 * {@inheritDoc}
	 */
	@Override
	public String getName() {
		return NAME;
	}

	public void addUploadRequestListener(UploadRequestListener uploadRequestListener) {
		uploadRequestListeners.add(uploadRequestListener);
	}

	public void removeUploadRequestListener(UploadRequestListener uploadRequestListener) {
		uploadRequestListeners.remove(uploadRequestListener);
	}

	private void fireUploadRequestAdded(Client remoteClient, UploadRequest uploadRequest) {
		for (UploadRequestListener uploadRequestListener: uploadRequestListeners) {
			uploadRequestListener.uploadRequestAdded(remoteClient, uploadRequest);
		}
	}

	private void fireUploadRequestUpdated(Client remoteClient, UploadRequest uploadRequest) {
		for (UploadRequestListener uploadRequestListener: uploadRequestListeners) {
			uploadRequestListener.uploadRequestUpdated(remoteClient, uploadRequest);
		}
	}

	private void fireUploadRequestRemoved(Client remoteClient, UploadRequest uploadRequest) {
		for (UploadRequestListener uploadRequestListener: uploadRequestListeners) {
			uploadRequestListener.uploadRequestRemoved(remoteClient, uploadRequest);
		}
	}

	public void addClientUploadRequests(Client remoteClient, UploadRequest uploadRequest) {
		if (clientUploadRequests.put(remoteClient, uploadRequest) != null) {
			fireUploadRequestUpdated(remoteClient, uploadRequest);
		} else {
			fireUploadRequestAdded(remoteClient, uploadRequest);
		}
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void init() {
		super.init();
		uploadRequestListeners = new HashSet<UploadRequestListener>();
		clientUploadRequests = new HashMap<Client, UploadRequest>();
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void destroy() {
		uploadRequestListeners.clear();
		uploadRequestListeners = null;
		clientUploadRequests.clear();
		clientUploadRequests = null;
		super.destroy();
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	protected void work() {
		Iterator<Client> remoteClients = clientUploadRequests.keySet().iterator();
		List<Client> clientsToRemove = new ArrayList<Client>();
		long now = System.currentTimeMillis();
		while (remoteClients.hasNext()) {
			Client remoteClient = remoteClients.next();
			UploadRequest uploadRequest = clientUploadRequests.get(remoteClient);
			if ((now - uploadRequest.getTimestamp()) > (2 * 60 * 60 * 1000)) {
				clientsToRemove.add(remoteClient);
			}
		}
		for (Client clientToRemove: clientsToRemove) {
			UploadRequest uploadRequest = clientUploadRequests.remove(clientToRemove);
			fireUploadRequestRemoved(clientToRemove, uploadRequest);
		}
	}

}
