/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */

package freemule.gui;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.swing.table.AbstractTableModel;

import freemule.application.Client;

/**
 * @author <a href="mailto:droden@gmail.com">David Roden</a>
 * @version $Id: SearchResultTableModel.java 130 2006-02-18 18:27:11Z bombe $
 */
public class SearchResultTableModel extends AbstractTableModel {

	private static final String[] columnNames = new String[] { "File", "Size", "Availability", "Hash" };
	private static final Class[] columnClasses = new Class[] { String.class, Long.class, Integer.class, String.class };

	private List<ResultFile> files = new ArrayList<ResultFile>();
	private Map<String, List<Client>> fileNameClients = new HashMap<String, List<Client>>();
	private Map<String, List<String>> fileNames = new HashMap<String, List<String>>();
	private Map<String, List<Client>> fileCommentClients = new HashMap<String, List<Client>>();
	private Map<String, List<String>> fileComments = new HashMap<String, List<String>>();
	private Map<String, List<Integer>> fileRatings = new HashMap<String, List<Integer>>();

	public void addFile(ResultFile resultFile) {
		String fileKey = resultFile.getKey();
		if (files.contains(resultFile)) {
			int index = files.indexOf(resultFile);
			files.get(index).setAvailability(files.get(index).getAvailability() + 1);
		} else {
			files.add(resultFile);
		}
		List<Client> clients = fileNameClients.get(fileKey);
		if (clients == null) {
			clients = new ArrayList<Client>();
			fileNameClients.put(fileKey, clients);
		}
		clients.add(resultFile.getClient());
		List<String> names = fileNames.get(fileKey);
		if (names == null) {
			names = new ArrayList<String>();
			fileNames.put(fileKey, names);
		}
		names.add(resultFile.getName());
		String comment = resultFile.getComment();
		int rating = resultFile.getRating();
		Client client = resultFile.getClient();
		if (((comment != null) && (comment.length() != 0)) || (rating > 0)) {
			List<String> comments = fileComments.get(fileKey);
			if (comments == null) {
				comments = new ArrayList<String>();
				fileComments.put(fileKey, comments);
			}
			List<Integer> ratings = fileRatings.get(fileKey);
			if (ratings == null) {
				ratings = new ArrayList<Integer>();
				fileRatings.put(fileKey, ratings);
			}
			List<Client> commentClients = fileCommentClients.get(fileKey);
			if (commentClients == null) {
				commentClients = new ArrayList<Client>();
				fileCommentClients.put(fileKey, commentClients);
			}
			comments.add(comment);
			ratings.add(rating);
			commentClients.add(client);
		}
	}

	public String[] getFilenames(String key) {
		List<String> names = fileNames.get(key.startsWith("freenet:") ? key.substring(8) : key);
		if (names != null) {
			return names.toArray(new String[names.size()]);
		}
		return new String[0];
	}

	public boolean hasComments(String key) {
		return (fileComments.get(key) != null);
	}

	public Client[] getCommentClients(String key) {
		List<Client> commentClients = fileCommentClients.get(key);
		if (commentClients != null) {
			return commentClients.toArray(new Client[commentClients.size()]);
		}
		return new Client[0];
	}

	public String[] getComments(String key) {
		List<String> comments = fileComments.get(key);
		if (comments != null) {
			return comments.toArray(new String[comments.size()]);
		}
		return new String[0];
	}

	public int[] getRatings(String key) {
		List<Integer> ratings = fileRatings.get(key);
		if (ratings != null) {
			Integer[] allRatings = ratings.toArray(new Integer[ratings.size()]);
			int[] result = new int[allRatings.length];
			int index = 0;
			for (int rating: allRatings) {
				result[index++] = rating;
			}
			return result;
		}
		return new int[0];
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public String getColumnName(int column) {
		return columnNames[column];
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public Class< ? > getColumnClass(int columnIndex) {
		return columnClasses[columnIndex];
	}

	/**
	 * {@inheritDoc}
	 */
	public int getRowCount() {
		return files.size();
	}

	/**
	 * {@inheritDoc}
	 */
	public int getColumnCount() {
		return columnNames.length;
	}

	/**
	 * {@inheritDoc}
	 */
	public Object getValueAt(int rowIndex, int columnIndex) {
		ResultFile resultFile = files.get(rowIndex);
		switch (columnIndex) {
		case -1:
			return resultFile;
		case 0:
			return resultFile.getName();
		case 1:
			return new Long(resultFile.getLength());
		case 2:
			return new Integer(resultFile.getAvailability());
		case 3:
			return resultFile.getKey();
		}
		return null;
	}

	public static class ResultFile {

		private final Client client;
		private final String name;
		private final long length;
		private final String key;
		private final String comment;
		private final int rating;
		private int availability;
		private final int blockCount;

		public ResultFile(Client client, String name, long length, String key, String comment, int rating, int blockCount) {
			this.client = client;
			this.name = name;
			this.length = length;
			this.key = key.startsWith("freenet:") ? key.substring(8) : key;
			this.comment = comment;
			this.rating = rating;
			this.availability = 1;
			this.blockCount = blockCount;
		}

		/**
		 * @return Returns the client.
		 */
		public Client getClient() {
			return client;
		}

		/**
		 * @return Returns the availability.
		 */
		public int getAvailability() {
			return availability;
		}

		/**
		 * @param availability
		 *            The availability to set.
		 */
		public void setAvailability(int availability) {
			this.availability = availability;
		}

		/**
		 * @return Returns the hash.
		 */
		public String getKey() {
			return key;
		}

		/**
		 * @return Returns the length.
		 */
		public long getLength() {
			return length;
		}

		/**
		 * @return Returns the name.
		 */
		public String getName() {
			return name;
		}

		public boolean equals(Object o) {
			if ((o == null) || !(o instanceof ResultFile)) {
				return false;
			}
			ResultFile resultFile = (ResultFile) o;
			return resultFile.key.equals(key);
		}

		public int hashCode() {
			return key.hashCode();
		}

		/**
		 * @return Returns the comment.
		 */
		public String getComment() {
			return comment;
		}

		/**
		 * @return Returns the rating.
		 */
		public int getRating() {
			return rating;
		}

		
		/**
		 * @return Returns the blockCount.
		 */
		public int getBlockCount() {
			return blockCount;
		}

	}

}
