/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */

package freemule.io;

import java.io.InputStream;
import java.util.Arrays;

/**
 * A special {@link InputStream} that always returns <code>0</code>. Useful
 * for padding segments to the required size (in conjunction with
 * {@link java.io.SequenceInputStream}).
 * 
 * @author <a href="mailto:droden@gmail.com">David Roden</a>
 * @version $Id: ZeroInputStream.java 130 2006-02-18 18:27:11Z bombe $
 */
public class ZeroInputStream extends InputStream {

	/**
	 * The length of the input stream.
	 */
	private long length;

	/**
	 * Creates a ZeroInputStream that will deliver <code>length</code> zero
	 * bytes.
	 * 
	 * @param length
	 *            The length of the stream
	 */
	public ZeroInputStream(long length) {
		this.length = length;
	}

	/**
	 * Returns the number of bytes that are still available in this stream.
	 */
	@Override
	public int available() {
		return (int) Math.min(Integer.MAX_VALUE, length);
	}

	/**
	 * Skips <code>n</code> bytes of input.
	 * 
	 * @return The number of bytes that have been skipped
	 */
	@Override
	public long skip(long n) {
		if (length <= 0) {
			return 0;
		}
		long r = Math.min(n, length);
		length -= Math.min(n, length);
		return r;
	}

	/**
	 * Reads a single zero from the stream. Returns <code>-1</code> if the end
	 * of the stream has been reached.
	 * 
	 * @return <code>0</code>, or <code>-1</code> if the end of the stream
	 *         has been reached
	 */
	@Override
	public int read() {
		if (length <= 0) {
			return -1;
		}
		length--;
		return 0;
	}

	/**
	 * Fills <code>buffer</code> with content from the stream. Calling the
	 * method is the same as calling
	 * {@link #read(byte[], int, int) read(buffer, 0, buffer.length)}.
	 * 
	 * @return The number of bytes that have been read, or <code>-1</code> if
	 *         the end of the stream has been reached
	 */
	@Override
	public int read(byte[] buffer) {
		return read(buffer, 0, buffer.length);
	}

	/**
	 * Fills <code>buffer</code> with content from the stream. If not enough
	 * content is available, only a part of <code>buffer</code> will be
	 * filled.
	 * 
	 * @return The number of bytes that have been read, or <code>-1</code> if
	 *         the end of the stream has been reached.
	 */
	@Override
	public int read(byte[] b, int off, int len) {
		if (length <= 0) {
			return -1;
		}
		int r = (int) Math.min(Integer.MAX_VALUE, Math.min(len, length));
		length -= r;
		Arrays.fill(b, off, off + r, (byte) 0);
		return r;
	}

}
