/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA.
 */

package freenet.fcp;

import java.io.EOFException;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.List;

/**
 * @author <a href="mailto:droden@gmail.com">David Roden</a>
 * @version $Id: FcpFecDecodeSegment.java,v 1.1 2005/11/28 22:54:44 bombe Exp $
 */
public class FcpFecDecodeSegment extends FcpCommand {

	private SegmentHeader segmentHeader;
	private List dataBlocks;
	private List checkBlocks;
	private List requestedBlocks;
	private InputStream dataInput;
	private InputStream checkInput;

	private int blockCount;
	private int blockSize;
	private long bytesReceived;
	private File tempFile;
	private OutputStream dataOutput;

	/**
	 * 
	 */
	public FcpFecDecodeSegment(SegmentHeader segmentHeader, List dataBlocks, List checkBlocks, InputStream dataInput, InputStream checkInput) {
		this.segmentHeader = segmentHeader;
		this.dataBlocks = dataBlocks;
		this.checkBlocks = checkBlocks;
		this.dataInput = dataInput;
		this.checkInput = checkInput;
		// calculate required blocks
		this.requestedBlocks = new ArrayList();
		for (int index = 0; index < segmentHeader.getBlockCount(); index++) {
			if (!dataBlocks.contains(new Integer(index))) {
				requestedBlocks.add(new Integer(index));
			}
		}
		for (int index = 0; index < segmentHeader.getCheckBlockCount(); index++) {
			if (!checkBlocks.contains(new Integer(segmentHeader.getBlockCount() + index))) {
				requestedBlocks.add(new Integer(segmentHeader.getBlockCount() + index));
			}
		}
	}

	public FcpFecDecodeSegment(SegmentHeader segmentHeader, int[] dataBlocks, int[] checkBlocks, InputStream dataInput, InputStream checkInput) {
		this.segmentHeader = segmentHeader;
		this.dataInput = dataInput;
		this.checkInput = checkInput;
		this.dataBlocks = new ArrayList();
		for (int index = 0, size = dataBlocks.length; index < size; index++) {
			this.dataBlocks.add(new Integer(dataBlocks[index]));
		}
		this.checkBlocks = new ArrayList();
		for (int index = 0, size = checkBlocks.length; index < size; index++) {
			this.checkBlocks.add(new Integer(checkBlocks[index]));
		}
		// calculate required blocks
		this.requestedBlocks = new ArrayList();
		for (int index = 0; index < segmentHeader.getBlocksRequired(); index++) {
			if (!this.dataBlocks.contains(new Integer(index))) {
				requestedBlocks.add(new Integer(index));
			}
		}
		for (int index = 0; index < segmentHeader.getCheckBlockCount(); index++) {
			if (!this.checkBlocks.contains(new Integer(segmentHeader.getBlocksRequired() + index))) {
				requestedBlocks.add(new Integer(segmentHeader.getBlocksRequired() + index));
			}
		}
	}

	/**
	 * @return Returns the blockCount.
	 */
	public int getBlockCount() {
		return blockCount;
	}

	/**
	 * @return Returns the blockSize.
	 */
	public int getBlockSize() {
		return blockSize;
	}

	public int getRequestedBlock(int index) {
		return ((Integer) requestedBlocks.get(index)).intValue();
	}

	public InputStream getDecodedData() {
		try {
			return new TemporaryFileInputStream(tempFile);
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
		}
		return null;
	}

	/**
	 * {@inheritDoc}
	 */
	public String getName() {
		return "FECDecodeSegment";
	}

	/**
	 * {@inheritDoc}
	 */
	protected boolean writeCommand(OutputStream outputStream) throws IOException {
		byte[] segmentHeaderData = segmentHeader.toByteArray();
		boolean hasLastDataBlock = (segmentHeader.getSegment() == (segmentHeader.getSegmentCount() - 1)) && ((Integer) dataBlocks.get(dataBlocks.size() - 1)).intValue() == (segmentHeader.getBlocksRequired() - 1);
		outputStream.write(("DataLength=" + encodeHex(dataBlocks.size() * segmentHeader.getBlockSize() + checkBlocks.size() * segmentHeader.getCheckBlockSize() + segmentHeaderData.length) + "\r\n").getBytes());
		outputStream.write(("BlockList=" + encodeList(dataBlocks) + "\r\n").getBytes());
		outputStream.write(("CheckList=" + encodeList(checkBlocks) + "\r\n").getBytes());
		outputStream.write(("RequestedList=" + encodeList(requestedBlocks) + "\r\n").getBytes());
		outputStream.write(("Data\r\n").getBytes());
		outputStream.write(segmentHeaderData);
		int dataBlocksSize = dataBlocks.size() * segmentHeader.getBlockSize();
		int remaining = (int) (dataBlocksSize - (hasLastDataBlock ? segmentHeader.getBlockSize() - (segmentHeader.getFileLength() % segmentHeader.getBlockSize()): 0));
		copyStream(outputStream, dataInput, remaining);
		if (hasLastDataBlock) {
			remaining = (int) (segmentHeader.getBlockSize() - (segmentHeader.getFileLength() % segmentHeader.getBlockSize()));
			byte[] buffer = new byte[256 * 1024];
			while (remaining > 0) {
				outputStream.write(buffer, 0, Math.min(256 * 1024, remaining));
				remaining -= 256 * 1024;
			}
		}
		int checkBlocksSize = checkBlocks.size() * segmentHeader.getCheckBlockSize();
		copyStream(outputStream, checkInput, checkBlocksSize);
		return false;
	}

	/**
	 * {@inheritDoc}
	 */
	protected void handleFcpMessage(FcpMessage fcpMessage) throws FcpException, IOException {
		if ("BlocksDecoded".equals(fcpMessage.getName())) {
			blockCount = (int) parseHex(fcpMessage.getParameter("BlockCount"));
			blockSize = (int) parseHex(fcpMessage.getParameter("BlockSize"));
			bytesReceived = 0;
			tempFile = File.createTempFile("freenet-fcp.", ".data");
			dataOutput = new FileOutputStream(tempFile);
		} else if ("Restarted".equals(fcpMessage.getName())) {
			bytesReceived = 0;
			dataOutput.close();
			dataOutput = new FileOutputStream(tempFile);
		} else if ("DataChunk".equals(fcpMessage.getName())) {
			dataOutput.write(fcpMessage.getData());
			bytesReceived += fcpMessage.getData().length;
			complete = (bytesReceived == (blockCount * blockSize));
			if (complete) {
				dataOutput.close();
			}
		}
	}

	public static class TemporaryFileInputStream extends FileInputStream {

		private File tempFile;

		public TemporaryFileInputStream(File tempFile) throws FileNotFoundException {
			super(tempFile);
			this.tempFile = tempFile;
		}

		public void close() {
			tempFile.delete();
		}

	}

}
